
/* Main namespace. */
com.extrabux.extensions.Extrabux = (function (my) {

  var $ = my.$;

  var NS = {};

  Components.utils.import("resource://extrabux/modules/logging.jsm", NS);
  Components.utils.import("resource://extrabux/modules/config.jsm", NS);
  Components.utils.import("resource://extrabux/modules/underscore.jsm", NS);
  Components.utils.import("resource://extrabux/modules/storage.jsm", NS);
  Components.utils.import("resource://extrabux/modules/utils.jsm", NS);
  Components.utils.import("resource://extrabux/modules/uninstaller.jsm", NS);

  var _ = NS._;


  /* Import Controllers. */
  var controllersNS = {};
  Components.utils.import("resource://extrabux/modules/mainController.jsm", controllersNS);
  Components.utils.import("resource://extrabux/modules/userAuthController.jsm", controllersNS);
  Components.utils.import("resource://extrabux/modules/couponListController.jsm", controllersNS);


  /* Import models. */
  var modelsNS = {};
  Components.utils.import("resource://extrabux/modules/storeCache.jsm", modelsNS);
  Components.utils.import("resource://extrabux/modules/store.jsm", modelsNS);
  Components.utils.import("resource://extrabux/modules/user.jsm", modelsNS);
  Components.utils.import("resource://extrabux/modules/coupon.jsm", modelsNS);
  Components.utils.import("resource://extrabux/modules/click.jsm", modelsNS);


  /* Import views. */
  var viewsNS = {};
  Components.utils.import("resource://extrabux/modules/notificationBar.jsm", viewsNS);
  Components.utils.import("resource://extrabux/modules/userAuth.jsm", viewsNS);
  Components.utils.import("resource://extrabux/modules/couponList.jsm", viewsNS);
  Components.utils.import("resource://extrabux/modules/restrictions.jsm", viewsNS);

  var lastSelectedTab = null;


  function _onPageLoaded(event) {
    var browser = gBrowser.getBrowserForDocument(event.target);
    if (!browser) {
      return;
    }

    var pageIsFrame = (event.target instanceof Ci.nsIDOMHTMLDocument &&
      event.target != browser.contentDocument);
    if (pageIsFrame) {
      // Not interested in frames.
      return;
    }

    // Add the tab browser to the list.
    var info = my.getMVCForBrowserTab(browser);

    NS.logger.debug("_onPageLoaded: " + browser.contentWindow.location);

    info.controllers.Main.pageLoaded(browser);
  }


  function _onTabSelect(event) {
    if (lastSelectedTab) {
      // Close panels of the unselected tab.
      var browser_last = gBrowser.getBrowserForTab(lastSelectedTab);
      var info_last = my.getMVCForBrowserTab(browser_last);
      info_last.controllers.Main.tabUnselected();
    }
    lastSelectedTab = event.target;
  }


  function _onTabClose(event) {
    var browser = gBrowser.getBrowserForTab(event.target);
    var info = my.getMVCForBrowserTab(browser);

    // Destroy all XUL elements.
    _.each(info.views, function(view, name) {
      NS.logger.debug("TabClose -> destroying view: " + name);
      view.destroy();
    });

    _.each(info.controllers, function(c) {
      c.tabClosed();
    });
    // Remove the tab browser from the list.
    my.tabInfo = _.without(my.tabInfo, info);

    if (lastSelectedTab === event.target) {
      lastSelectedTab = null;
    }
  }


  function _onTabOpen(event) {
    var browser = gBrowser.getBrowserForTab(event.target);
    // Create the MVC object for this tab.
    my.getMVCForBrowserTab(browser);
  }


  function _clickedOutsidePanel(browser, event) {
    var MVCInfo = my.getMVCForBrowserTab(browser);
    MVCInfo.controllers.Main.clickedOutsidePanel(event);
  }


  function _onTabRestore(event) {
    var browser = gBrowser.getBrowserForTab(event.target);
    var MVCInfo = my.getMVCForBrowserTab(browser);
    MVCInfo.controllers.Main.restoredTab = true;
  }


  function _onExamineResponse(httpChannel, browser) {
    if (httpChannel.responseStatus >= 300 && httpChannel.responseStatus < 400) {
      var MVCInfo = my.getMVCForBrowserTab(browser);
      MVCInfo.controllers.Main.handleRedirect(httpChannel.URI.spec);
    }
  }


  /******* Public functions. *******/

  /** Returns the MVC object (i.e. a wrapper object that has references
   *  to MVC components) for a tab.
   *  If the MVC wrapper does not exist, creates it.
   *  @param browser: the <browser> object (for the respective tab).
   */
  my.getMVCForBrowserTab = function(browser) {

    // TODO(Tom): That's probably not necessary, wee could use selectedBrowser.
    browser = browser || gBrowser.getBrowserForDocument(gBrowser.contentDocument);

    // Search for the MVC object.
    var tabInfo = _.detect(my.tabInfo, function(item) {
      if (item.browser === browser) {
        return item;
      }
    });

    if (tabInfo) {
      // Found it.
      return tabInfo;
    } else {
      // Not found. Create a new one and store it.
      var info = {
        browser : browser
      };

      info.models = {
        Store : new modelsNS.Store(),
        Coupon: new modelsNS.Coupon(),
        Click: new modelsNS.Click()
      };

      info.views = {
        NotificationBar: new viewsNS.NotificationBar(window, browser, info.models, $),
        CouponList: new viewsNS.CouponList(window, browser, info.models, $),
        UserAuth: new viewsNS.UserAuth(window, browser, info.models, $),
        Restrictions: new viewsNS.Restrictions(window, browser, info.models, $)
      };

      info.controllers = {
        UserAuth : new controllersNS.UserAuth(window, browser, info.models, info.views, $),
        CouponList : new controllersNS.CouponList(window, browser, info.models, info.views, $),
        Main: new controllersNS.Main(window, browser, info.models, info.views, $)
      };

      // Inject the MVC components.
      // TODO(Tom): Ugly. There must be a better way...
      _.each(info.views, function(view) {
        view._controllers = info.controllers;
      });
      _.each(info.views, function(view) {
        view._views = info.views;
      });
      _.each(info.controllers, function(controller) {
        controller._controllers = info.controllers;
      });

      my.tabInfo.push(info);

      browser.addEventListener("click", function(event) {
        // Close the panels when user clicks outside the panel.
        _clickedOutsidePanel(browser, event);
      }, false);

      browser.addEventListener("DOMContentLoaded", _onPageLoaded, false);

      return info;
    }
  };


  my._showWelcome = function() {
    window.setTimeout(function() {
      // We have to use a timeout, otherwise the tab may get replaced
      // by FF initial tab.
      gBrowser.selectedTab = gBrowser.addTab(NS.Config.welcomeURL);
    }, 2000);
  };


  function getBrowserFromChannel(aChannel) {
    try {
      var notificationCallbacks =
        aChannel.notificationCallbacks ?
          aChannel.notificationCallbacks :
          aChannel.loadGroup.notificationCallbacks;

      if (!notificationCallbacks)
        return null;

      var domWin = notificationCallbacks.getInterface(Ci.nsIDOMWindow);
      return gBrowser.getBrowserForDocument(domWin.top.document);
    }
    catch (e) {
      return null;
    }
  }


  var _httpObserver = {
    observe: function(subject, topic, data) {
      if (topic == "http-on-examine-response") {
        var channel = subject.QueryInterface(Ci.nsIHttpChannel);
        var browser = getBrowserFromChannel(channel);
        browser !== null && _onExamineResponse(channel, browser);
      }
    }
  };


  my.initialize = function(event) {
    // Try to load cache from local DB.
    modelsNS.GlobalStoreCache.reloadCacheLocally();
    // Start up the timeout for updates from server.
    modelsNS.GlobalStoreCache.initializeUpdating();

    var container = gBrowser.tabContainer;
    container.addEventListener("TabClose", _onTabClose, false);
    container.addEventListener("TabSelect", _onTabSelect, false);
    container.addEventListener("TabOpen",   _onTabOpen,   false);

    /* Create MVC objects for restored tabs (+ set a 'restored'
     * flag for them) */
    document.addEventListener("SSTabRestoring", _onTabRestore, false);

    // Create MVC objects for the first tab when opening a new window
    // (none of the above events is fired for the first tab of a new window).
    var browser = gBrowser.getBrowserForDocument(event.target);
    my.getMVCForBrowserTab(browser);

    // We need to observe redirects (there can be special query params that
    // we need to look for).
    var observerService = Cc["@mozilla.org/observer-service;1"].getService(
      Ci.nsIObserverService);
    observerService.addObserver(
      _httpObserver, "http-on-examine-response", false);

    var self = this;

    /* Open welcome screen if this is the first time user runs FF after
     * installing the extension. */
    var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
    prefs = prefs.getBranch("extensions.extrabux@extrabux.com.");

    var welcomeShown = false;
    if (prefs.getBoolPref("showWelcome")) {
      this._showWelcome();
      prefs.setBoolPref("showWelcome", false);
      welcomeShown = true;
    }
    NS.Utils.getAddonVersionAsync(function(currentVersion) {
      var prefs = NS.Utils.getPrefs();
      var lastKnown = prefs.getCharPref("lastKnownVersion");
      NS.logger.debug("Overlay -> last known add-on version: " + lastKnown);
      if ( ! welcomeShown && lastKnown < currentVersion) {
        // Show update.
        NS.logger.debug("Overlay -> showing add-on updated screen");
        self._showWelcome();
      }
      prefs.setCharPref("lastKnownVersion", currentVersion);
    });

    modelsNS.GlobalUser.initializeLoginManagers(window, $);
    modelsNS.GlobalUser.handleInitialLogin();

    NS.GlobalUninstaller.initialize(gBrowser);

    return;
  };


  my.finalize = function(event) {
    _.each(my.tabInfo, function(mvc) {
      _.each(mvc.controllers, function(c) {
        c.tabClosed();
      });
      _.each(mvc.views, function(v, name) {
        NS.logger.debug("Overlay -> finalize -> destroying view: " + name);
        v.destroy();
      });
    });
    my.tabInfo = [];
    var observerService = Cc["@mozilla.org/observer-service;1"].getService(
      Ci.nsIObserverService);
    observerService.removeObserver(_httpObserver, "http-on-examine-response");
  };


  // Stores all the MVC components (one for each tab).
  my.tabInfo = [];

  return my;

})(com.extrabux.extensions || {});

window.addEventListener("load", function (event) { com.extrabux.extensions.Extrabux.initialize(event); }, false);
window.addEventListener("unload", function (event) { com.extrabux.extensions.Extrabux.finalize(event); }, false);

