var EXPORTED_SYMBOLS = ["Coupon"];


var Ci = Components.interfaces;
var Cc = Components.classes;
var Cu = Components.utils;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);


Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/event.jsm");
Cu.import("resource://extrabux/modules/underscore.jsm");


Cu.import("resource://extrabux/modules/xmlHttpPaginator.jsm");
Cu.import("resource://extrabux/modules/config.jsm");
Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;


var baseURL = Config.baseURL + "coupon/";


/*
* The model for coupons.
* Wraps the communication with the Extrabux API.
*/

function Coupon() {
  // List of coupons that the currently active tab is dealing with.
  this.activeCoupons = [];

  this.listLoaded = new Event(this);
  this.listLoading = new Event(this);
}


/* Inherit the HTTP and list downloading voodoo. */
InheritFrom(XMLHTTPPaginator, Coupon);


/** Download the list of coupons from the server.
 */
Coupon.prototype.loadListForMerchantId = function(merchantId, callbackObj) {
  //Delete the old coupons.
  this.activeCoupons = [];
  this.listLoading.notify();

  var self = this;
  this._downloadList(baseURL, {"merchant_id": merchantId},
    "total_coupons", "coupons", "coupon", null,
    {"on": {
      "success": function(coupons, finished) {
        if ( ! finished) {
          // We're only interested in the whole list.
          return;
        }
        // Only select coupons that have already started.
        self.activeCoupons = _.select(coupons, function(coupon) {
          var d = Utils.getCouponDateFromString(_txt(coupon.start_date));
          return (d <= Date.now());
        });
        self.listLoaded.notify(coupons);
        callbackObj.on.success(coupons);
      },
      "error": callbackObj.on.error
    }});
};
