var EXPORTED_SYMBOLS = ["CouponList"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;
Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/genericView.jsm");


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu.import("resource://extrabux/modules/underscore.jsm");

Cu.import("resource://extrabux/modules/config.jsm");


function CouponList(window, browser, models, jquery) {
  CouponList.baseConstructor.call(this, window, browser, models, jquery);

  var self = this;
  this._models.Coupon.listLoaded.attach(function(who, coupons) {
    self.listLoaded(coupons);
  });

  this._models.Coupon.listLoading.attach(function(who) {
    self.listLoading();
  });

  this._panel = this._cloneTemplatePanel("#extrabux-couponList-panel-template");
}


InheritFrom(GenericView, CouponList);


/**
 * Generate the DOM element for coupon. It's going to be appended to the
 * respective coupon container.
 * @templateElem: Element to be cloned as a template for coupons.
 * @coupon: JSON object (represents coupon retrieved from the API).
 */
CouponList.prototype.generateCouponElement = function(templateElem, coupon) {
  var couponElem = templateElem.clone(true);

  couponElem.find(".extrabux-coupon-detail-info-1").text(_txt(coupon.name));

  var cashback = Utils.getCashbackString(coupon.store);

  var stringsBundle = this.$("#extrabux-strings").get(0);

  if (_txt(coupon.store.cashback_flat) != "0" || _txt(coupon.store.cashback_percent) != "0") {
    var cashbackMsg = _.sprintf(stringsBundle.getString("cashback-message"), cashback);
    couponElem.find(".extrabux-coupon-detail-info-2").text(cashbackMsg);
  }

  var expireStr = Utils.formatExpirationString(_txt(coupon.end_date), stringsBundle);
  var expireMsg = _.sprintf(expireStr + " %s",
    _txt(coupon.restrictions) || "");
  couponElem.find(".extrabux-coupon-detail-info-3").text(expireMsg);

  var btn = couponElem.find(".extrabux-coupon-button");
  var code = _txt(coupon.code);

  if (code) {
    btn.text(stringsBundle.getString("code-button-label"));
  } else {
    btn.text(stringsBundle.getString("activate-button-label"));
  }
  // Set the id, so that we know what coupon has been clicked.
  btn.attr("id", _txt(coupon.id));

  couponElem.attr("hidden", false);

  return couponElem;
};


CouponList.prototype.replaceCouponButtonByCode = function(couponId, couponCode) {
  var btn = this._panel.find(".extrabux-coupon-button[id='" + couponId + "']");
  btn.hide();
  var container = btn.parents(".extrabux-coupon-right");
  var codeLabel = container.find(".extrabux-coupon-code-label");
  codeLabel.attr("hidden", false);
  codeLabel.text(couponCode);
}


CouponList.prototype._clearList = function() {
  var containerElem = this._panel.find(":first");
  // Get rid of the old coupons (but keep the first one,
  // that's the coupon template).
  containerElem.children().not(":first-child").remove();
};


/**
 * The coupons list for this tab has been (re)loaded => update the coupons.
 */
CouponList.prototype.listLoaded = function(_coupons) {
  this._clearList();
  var containerElem = this._panel.find(":first");
  // Create the new coupons.
  var templateElem = containerElem.children(".extrabux-coupon-container");

  // Use activeCoupons instead of @coupons (we only want coupons
  // that have already started).
  var activeCoupons = this._models.Coupon.activeCoupons;
  _.each(activeCoupons, function(item) {
    var elem = this.generateCouponElement(templateElem, item);
    containerElem.append(elem);
  }, this);
};


CouponList.prototype.listLoading = function() {
  //TODO(Tom): Display some "loading..." message.
};


CouponList.prototype.show = function() {
  this._panel.attr("hidden", false);
  var btns = this._views.NotificationBar.getButtons();
  this._panel.get(0).openPopup(btns.coupon.get(0), "after_start", -50, 0);

  var stringsBundle = this.$("#extrabux-strings").get(0);
  btns.coupon.children(":first").attr("value", stringsBundle.getString("coupons-btn-open"));
  btns.coupon.children(":first").next().attr("src", "chrome://extrabux/skin/dropup-icon.png");

  this._adjustCouponListHeight();

  this._visible = true;
};


/**
 * Adjusts coupon list panel height to fit the coupons (we want no empty
 * space under the coupons).
 */
CouponList.prototype._adjustCouponListHeight = function() {
  var self = this;
  var couponsHeight = 0;

  // Calculate the total height of the coupons.
  this._panel.children(".extrabux-couponList-container").children().each(
    function() {
      couponsHeight += Math.floor(self.$(this).get(0).scrollHeight);
    });

  var containerHeight = this._panel.children(
    ".extrabux-couponList-container").get(0).clientHeight;

  if (couponsHeight < containerHeight) {
    // Make the coupon list smaller.
    this._panel.children(".extrabux-couponList-container").attr(
      "height", couponsHeight * 1.15);
  } else {
    // TODO(Tom): Find a better way than to hardcode the template height here!
    this._panel.children(".extrabux-couponList-container").attr("height", 380);
  }
};


CouponList.prototype.hide = function() {
  if ( ! this._visible) {
    // Don't hide what's already hidden.
    return;
  }
  var btns = this._views.NotificationBar.getButtons();
  var stringsBundle = this.$("#extrabux-strings").get(0);
  btns.coupon.children(":first").attr("value", stringsBundle.getString("coupons-btn-closed"));
  btns.coupon.children(":first").next().attr("src", "chrome://extrabux/skin/dropdown-icon.png");

  this._panel.get(0).hidePopup();
  this._visible = false;
};


CouponList.prototype.destroy = function() {
  this._panel.remove();
  this._panel = null;
};
