
var EXPORTED_SYMBOLS = ["CouponList"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);


Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;
Cu.import("resource://extrabux/modules/underscore.jsm");
Cu.import("resource://extrabux/modules/inheritFrom.jsm");


var models = {};
Cu.import("resource://extrabux/modules/user.jsm", models);
Cu.import("resource://extrabux/modules/httpWrapper.jsm");
Cu.import("resource://extrabux/modules/genericController.jsm");


function CouponList(window, browser, models, views, $) {
  CouponList.baseConstructor.call(this, window, browser, models, views, $);
}

InheritFrom(GenericController, CouponList);
_.extend(CouponList.prototype, HTTPWrapper.prototype);


CouponList.prototype.couponBtnClicked = function(btn) {
  // TODO: Get the button from view, not directly from the DOM.
  var id = this.$(btn).attr("id");
  this.handleCouponClicked(id);
};


CouponList.prototype.continueLinkClicked = function(elem) {
  var id = this.$(elem).attr("id");
  this.handleClickWorker(id);
};


CouponList.prototype.couponHeaderClicked = function(header) {
  var id = this.$(header).parent().parent().find(
    ".extrabux-coupon-button").attr("id");
  this.handleCouponClicked(id);
};


CouponList.prototype.handleCouponClicked = function(id) {
  // Hide coupon list (we don't want it to hang there when the user selected a coupon.)
  this._views.CouponList.hide();
  logger.debug("handleCouponClicked");

  var activeUser = models.GlobalUser.activeUser;

  this._controllers.UserAuth.delayedActions.clear();

  var self = this;
  var store = this._models.Store.activeStore;
  var noCashback = (_txt(store.cashback_flat) == "0" &&
    _txt(store.cashback_percent) == "0");

  // If there's no user logged in but there's no cashback, don't require user
  // to log in, otherwise require login.
  if ( ! activeUser && ! noCashback) {
    this._views.UserAuth.state = "signup";
    this._views.UserAuth.show(id);
    this._controllers.UserAuth.delayedActions.addAction(function() {
       self._controllers.CouponList.handleCouponClicked(id);
    });

    return;
  }

  this.handleClickWorker(id);
};


CouponList.prototype.handleClickWorker = function(id) {
  var self = this;

  var coupons = this._models.Coupon.activeCoupons;
  var clickedCoupon = _.detect(coupons, function(item) {
      return (_txt(item.id) == id);
  });

  this._views.NotificationBar.state = "couponActive";
  this._views.NotificationBar.show({coupon: clickedCoupon});

  var code = _txt(clickedCoupon.code);
  var couponId = _txt(clickedCoupon.id)
  if (code){
    this._views.CouponList.replaceCouponButtonByCode(couponId, code);
  }

  var params = {
    merchant_id: _txt(clickedCoupon.merchant_id),
    coupon_id: couponId
  };

  var activeUser = models.GlobalUser.activeUser;
  if (activeUser) {
    params.user_id = _txt(activeUser.email);
  }

  this._models.Click.generateClick(params, {on: {
    success: function(response) {
      self._get(_txt(clickedCoupon.link), {}, {on: {
        success: function() {},
        error: function() {}
      }}, true);
      self._redirectUserToCouponLink(_txt(response.click.merchant_transfer_url));
      logger.debug("Successfully generated coupon click: " + response.click.merchant_transfer_url);
    },
    error: function() {}
  }});
};


CouponList.prototype._redirectUserToCouponLink = function(url) {
  logger.debug("CouponList view -> couponsBtnClicked -> redirecting user to URL: " + url);
  this._browser.loadURI(url);
};
