var EXPORTED_SYMBOLS = ["ExtrabuxLoginManager"];


var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu["import"]("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;


Cu["import"]("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu["import"]("resource://extrabux/modules/underscore.jsm");

Cu["import"]("resource://extrabux/modules/event.jsm");
Cu["import"]("resource://extrabux/modules/inheritFrom.jsm");
Cu["import"]("resource://extrabux/modules/config.jsm");
Cu["import"]("resource://extrabux/modules/xmlHttpWrapper.jsm");


Cu["import"]("resource://extrabux/modules/genericLoginManager.jsm");
Cu["import"]("resource://extrabux/modules/sha1.jsm");


var baseURL = Config.baseURL + "user/";

function ExtrabuxLoginManager(browser, $) {
  logger.debug("Extrabux login manager constructor");
  ExtrabuxLoginManager.baseConstructor.call(this, browser, $);

  this.loginManager = LoginManager;

  this._id = "extrabux";
}


/* Inherit the HTTP voodoo. */
InheritFrom(GenericLoginManager, ExtrabuxLoginManager);


ExtrabuxLoginManager.prototype.__defineGetter__("id", function() {
  return this._id;
});


/*
 * Creates a new user on the remote server.
 * @param email: user's email. Must be escaped.
 * @param password: user's password. Must be escaped.
 */
ExtrabuxLoginManager.prototype.create = function(email, password, callbackObj) {
  var self = this;
  if (password.length < Config.minPwdLength) {
    callbackObj.on.error(
      _.sprintf("Passwords must be at least %d characters long.",
        Config.minPwdLength));
    return;
  }
  logger.debug("User model -> create user.");
  this._post(
    baseURL,
    {},
    {"email": email, "password": hex_sha1(password)},
    {"on": {
      "success": function(response) {
        logger.debug("User model -> create user success.");
        if (response.errors) {
          callbackObj.on.error(_txt(response.errors.error));
        } else {
          callbackObj.on.success();
        }
      },
      "error": function(reason) {
        logger.debug("User model -> creating new user with email " + email +
                     " failed. Reason: " + reason);
        callbackObj.on.error(reason);
      }
    }});
};


ExtrabuxLoginManager.prototype.logout = function() {
  this.superClass.logout.call(this);
  LoginManager.removeLogin();
};


/*
 * Authenticates a user on the remote server.
 * @param email: user's email. Must be escaped.
 * @param password: user's password. Must be escaped.
 */
ExtrabuxLoginManager.prototype.authenticate = function(email, password, callbackObj) {

  var creds = LoginManager.getLogin();

  if (creds) {
    email = email || creds.username;
    password = password || creds.password;
  }

  var self = this;
  this._get(
    baseURL + escape(email),
    {password: hex_sha1(password), publisher_id: Config.publisherId},
    {on: {
      success: function(response) {
        if (response.error) {
          logger.debug("Login error: " + _txt(response.error));
          callbackObj.on.error(_txt(response.error));
        } else {
          // Set the authenticated user as active.
          LoginManager.storeLogin(email, password);

          callbackObj.on.success(response.user);
        }
      },
      error: function(reason) {
        logger.debug("User model -> authentication failed for user with email " +
                     email + " failed. Reason: " + reason);
        callbackObj.on.error(reason);
      }
    }});
};


var LoginManager = {

  storeLogin: function(username, pwd) {
    var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
      Ci.nsILoginInfo, "init");
    var extLoginInfo = new nsLoginInfo(
      'chrome://extrabux', null, 'User Registration',
      username, pwd, "", "");

    var hostname = "chrome://extrabux";
    var formSubmitURL = null;
    var httprealm = 'User Registration';
    var loginManager = Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);

    var logins = loginManager.findLogins({}, hostname, formSubmitURL, httprealm);
    if (logins.length > 0) {
      loginManager.removeLogin(logins[0]);
    }
    loginManager.addLogin(extLoginInfo);
  },


  removeLogin: function() {
    var hostname = "chrome://extrabux";
    var formSubmitURL = null;
    var httprealm = 'User Registration';

    var loginManager = Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);
    var logins = loginManager.findLogins({}, hostname, formSubmitURL, httprealm);

    if (logins.length > 0) {
      loginManager.removeLogin(logins[0]);
    }
  },


  getLogin: function() {
    var hostname = "chrome://extrabux";
    var formSubmitURL = null;
    var httprealm = 'User Registration';

    try {
       // Get Login Manager
       var loginManager = Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);
       // Find users for the given parameters
       var logins = loginManager.findLogins({}, hostname, formSubmitURL, httprealm);
       if (logins.length === 0) {
         return null;
       } else {
         return {
           username: logins[0].username,
           password: logins[0].password
         };
       }
    }
    catch(e) {
      // This will only happen if there is no nsILoginManager component class
      logger.error("LoginManager -> getActiveLogin ->  no nsILoginManager component class");
    }
  }
};

