var EXPORTED_SYMBOLS = ["ExtrabuxWebsiteLoginManager"];


var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu["import"]("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;


Cu["import"]("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu["import"]("resource://extrabux/modules/underscore.jsm");

Cu["import"]("resource://extrabux/modules/inheritFrom.jsm");
Cu["import"]("resource://extrabux/modules/config.jsm");

Cu["import"]("resource://extrabux/modules/genericLoginManager.jsm");


var baseURL = Config.baseURL + "user/";

function ExtrabuxWebsiteLoginManager(browser, $) {
  ExtrabuxWebsiteLoginManager.baseConstructor.call(this, browser, $);

  this._id = "website";

  this.initialize();
}


/* Inherit the HTTP voodoo. */
InheritFrom(GenericLoginManager, ExtrabuxWebsiteLoginManager);


ExtrabuxWebsiteLoginManager.prototype.initialize = function() {
  var observerService = Cc["@mozilla.org/observer-service;1"].getService(
    Ci.nsIObserverService);
  observerService.addObserver(this, "cookie-changed", false);
};


ExtrabuxWebsiteLoginManager.prototype.observe = function(subject, topic, data) {
  if (data == "added") {
    var cookie = subject.QueryInterface(Ci.nsICookie);
    if (cookie.host == Config.homeSite && cookie.name == Config.loginCookieName) {
      var models = {};
      // We have to import User here (because of circular dependencies
      // if we injected it in the module-level scope).
      Cu["import"]("resource://extrabux/modules/user.jsm", models);
      if (models.GlobalUser.activeUser) {
        // Already logged in => do nothing.
        return;
      }
      if (models.GlobalUser.activeLoginManager) {
        // Currently using a different login manager -> do nothing.
        // If we did not handle this, we would have problems with FB
        // authentication, because the cookie might get set after the
        // FB login and thus "override" the FB login in the addon.
        return;
      }
      // The login cookie has been set and we're not logged in => log in.
      logger.debug("ExtrabuxWebsiteLoginManager -> cookie observed -> logging in");
      models.GlobalUser.activeLoginManager = "website";
      models.GlobalUser.authenticate(null, null, {on: {
        success: function() {
          logger.debug("ExtrabuxWebsiteLoginManager -> authenticated (cookie)");
        },
        error: function() {}
      }});
    }
  }
};


ExtrabuxWebsiteLoginManager.prototype.__defineGetter__("id", function() {
  return this._id;
});


ExtrabuxWebsiteLoginManager.prototype.authenticate = function(email, _password_, callbackObj) {
  var loginCookie = this._searchForLoginCookie();

  if ( ! loginCookie) {
    // TODO(Tom): We should not really return an error here, because this is
    // not an error per se. This calls for a bigger refactoring of the login
    // managers.
    callbackObj.on.error(null);
    return;
  }

  var jsonStr = atob(unescape(loginCookie.value));
  var authInfo = nativeJSON.decode(jsonStr);
  var email = authInfo.email;

  var user = {email: {"#text": email}};
  callbackObj.on.success(user);
  logger.debug("ExtrabuxWebsiteLoginManager -> auth success -> email: " + email);
};


ExtrabuxWebsiteLoginManager.prototype._searchForLoginCookie = function() {
  var cookieMgr = Cc["@mozilla.org/cookiemanager;1"].getService(Ci.nsICookieManager2);
  var loginCookie = null;

  for (var e = cookieMgr.getCookiesFromHost(Config.homeSite); e.hasMoreElements();) {
    var cookie = e.getNext().QueryInterface(Ci.nsICookie);
    if (cookie.name == Config.loginCookieName) {
      loginCookie = cookie;
    }
  }

  return loginCookie;
};


ExtrabuxWebsiteLoginManager.prototype.logout = function() {
  var self = this;

  // Carry out the generic logout routine.
  this.superClass.logout.call(this, {on: {
    success: function() {
      // Remove the cookie.
      var loginCookie = self._searchForLoginCookie();
      if (loginCookie) {
        // Remove the cookie so that we don't log in again automatically (which
        // would happen if we kept the cookie).
        var cookieMgr = Cc["@mozilla.org/cookiemanager;1"].getService(Ci.nsICookieManager);
        cookieMgr.remove(
          loginCookie.host, loginCookie.name, loginCookie.path, false);
        logger.debug("ExtrabuxWebsiteLoginManager -> logout -> cookie removed");
      }
    },
    error: function() {}
  }});
};
