var EXPORTED_SYMBOLS = [ "logger" ];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const INFO = 1;
const DEBUG = 2;
const WARNING = 4;
const ERROR = 5;

function logConsole(aMessage) {
  var consoleService = Cc["@mozilla.org/consoleservice;1"]
      .getService(Ci.nsIConsoleService);
  consoleService.logStringMessage(aMessage);
}

function noLog(msg, severity) {
  return;
}

Logger = function(logOutputFun, severityLevel) {
  if (logOutputFun == undefined) {
    this.logFun = logConsole;
  } else {
    this.logFun = logOutputFun;
  }
  this.severityLevel = severityLevel;
}

Logger.prototype = {
  info : function AL_info(msg) {
    this.log(msg, INFO);
  },

  debug : function AL_debug(msg) {
    this.log(msg, DEBUG);
  },

  warning : function AL_warning(msg) {
    this.log(msg, WARNING);
  },

  error : function AL_error(msg) {
    this.log(msg, ERROR);
  },

  setLogOutput : function(fun) {
    this.logFun = fun;
  },

  log: function(aMessage, severity) {
    var severityMsg = "";

    if (severity == undefined) {
      severity = INFO;
    }

    if (severity < this.severityLevel) {
      return;
    }

    switch (severity) {
      case INFO:
        severityMsg = "INFO: ";
        break;
      case DEBUG:
        severityMsg = "DEBUG: ";
        break;
      case WARNING:
        severityMsg = "WARNING: ";
        break;
      case ERROR:
        severityMsg = "ERROR: ";
        break;
      default:
        severityMsg = "INFO: ";
        break;
    }
    this.logFun("EXTRABUX " + severityMsg + aMessage + "\n");
  }
}


var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
try {
  var loggingEnabled = prefs.getBoolPref("extensions.extrabux.logging.enabled");
} catch (e) {
  loggingEnabled = false;
}

var logFun = null;
if (loggingEnabled) {
  logFun = dump;
} else {
  logFun = noLog;
}

var logger = new Logger(logFun, DEBUG);

