
var EXPORTED_SYMBOLS = ["Main"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu.import("resource://extrabux/modules/underscore.jsm");


Cu.import("resource://extrabux/modules/storeCache.jsm");


Cu.import("resource://extrabux/modules/config.jsm");
Cu.import("resource://extrabux/modules/httpWrapper.jsm");
Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/utils.jsm");
Cu.import("resource://extrabux/modules/delayedAction.jsm");
var _txt = Utils.text;


var models= {};
Cu.import("resource://extrabux/modules/user.jsm", models);
Cu.import("resource://extrabux/modules/genericController.jsm");


function Main(window, browser, models, views, jquery) {
  Main.baseConstructor.call(this, window, browser, models, views, jquery);

  this.lastClosed = null;

  this.currentURL = null;
  this.lastVisitedURL = null;

  // True iff the page has been opened at browser start (from session
  // store) and no other page has been loaded in this tab.
  this.restoredTab = false;

  this._redirectingFromExtrabuxSite = false;
  this._fakeReferrer = "";

  // Dont show any bar when we arrive at the store site (the
  // redirect URL that sent us to the store had a special
  // query param flag set).
  this._shouldIgnoreStore = false;

  this.delayedActions = new DelayedAction();
}


InheritFrom(GenericController, Main);
_.extend(Main.prototype, HTTPWrapper.prototype);


Main.prototype.onListLoaded = function() {
  if ( ! this._views.NotificationBar._visible) {
    // New stores have been loaded. If we're not showing anything, check
    // if we should show something now.
    this.lastVisitedURL = null;
    this.pageLoaded(this._browser);
  }
};


Main.prototype.handleRedirect = function(url) {
  var parts = _.words(url, "?");
  if (_isStoreBlockQueryParamSet(parts[1] || "")) {
    this._shouldIgnoreStore = true;
  }
};


function _isStoreBlockQueryParamSet(queryString) {
  var blockParamValue = Utils.getQueryParamByName(
    queryString, Config.queryParamForStoreBlocking);
  return (blockParamValue == "1");
}


function _isStoreHideAfterTimeoutParamSet(store) {
  return (_txt(store.network_id) == "1");
}


/* Extend the prototype.  */
Main.prototype.pageLoaded = function(browser) {
  var self = this;

  this.delayedActions.fire();

  this.currentURL = browser.contentWindow.location.toString();
  var currentHostname = Utils.getHostname(this.currentURL);

  if (Utils.URLDomainEquals(
      currentHostname, Utils.getHostname(this.lastVisitedURL))) {
    return;
  } else {
    // We're visiting a new site, hide all the views.
    _.each(this._views, function(view) {
      view.hide();
    });
    this._models.Click.storeLoaded();
  }

  this.lastVisitedURL = browser.contentWindow.location.toString();

  if (this._matchExtrabuxRedirectURL(this.currentURL)) {
    this._redirectingFromExtrabuxSite = true;
    // Set fake referrer (we cannot use document.referrer because of all
    // the redirects).
    this._fakeReferrer = this.currentURL;
  }

  this._controllers.UserAuth.delayedActions.clear();

  if (this._views.NotificationBar._visible) {
    this._views.UserAuth.hide();
    this._views.CouponList.hide();
    this._views.Restrictions.hide();
  }

  if (Utils.URLDomainEquals(this.lastClosed, currentHostname)) {
    return;
  }
  this.lastClosed = null;

  this._models.Store.updateActiveStore(currentHostname);
  var store = this._models.Store.activeStore;

  logger.debug("MainController -> pageLoaded -> store: " +
    nativeJSON.encode(store));

  var searchString = browser.contentWindow.location.search.toString();
  if (_isStoreBlockQueryParamSet(searchString)) {
    // Special query param set => ignore this store.
    this._fakeReferrer = "";
    return;
  }
  
  if ( ! store ) {
    logger.debug("current: " + browser.contentWindow.location);
    if (browser.contentWindow.location == Config.welcomeURL) {
      this._views.NotificationBar.state = "welcome";
      this._views.NotificationBar.show(showArg);
      this.delayedActions.addAction(function() {
        // We want to hide the welcome notification bar at next page load.
        self._views.NotificationBar.hide();
      });
    }
    // Either the store list was not yet loaded, or the current page
    // URL is not in the DB.
    return;
  }

  var showArg = {};
  if (_isStoreHideAfterTimeoutParamSet(store)) {
    showArg.hideTimeout = Config.notifBarHideTimeout;
  }

  if (this._shouldIgnoreStore) {
    // Query param was set during the redirect process => ignore this store.
    this._shouldIgnoreStore = false;
    return;
  }

  this._loadCouponsAndShowBar(store, this.restoredTab, this._fakeReferrer, showArg);

  // Mark tab as not restored (restored flag is only valid
  // until first page load).
  this.restoredTab = false;

  // We arrived at a store => unset redirect mode flag.
  this._redirectingFromExtrabuxSite = false;

  this._pingStoresWithImplicitCashback(store);
};


Main.prototype._pingStoresWithImplicitCashback = function(store) {
  var implicitCashbackStore = (_txt(store.plugin_auto_redirect) == "1")

  if ( ! implicitCashbackStore) {
    return;
  }

  // We are supposed to ping the merchant url when the store page is loaded.
  var loggedIn = (models.GlobalUser.activeUser !== null);
  this._generateClick(loggedIn);
};


/**
 * If the user comes from Config.redirectSite (i.e., www.extrabux.com), we
 * should not show normal Main state, but rather act as if he got cash
 * back / activated a coupon instead.
 */
Main.prototype.checkOverrideDefaultBehaviour = function(referrer, coupons, showArg) {
  var type = this._matchExtrabuxRedirectURL(referrer);
  if ( ! type) {
    return false;
  }

  if (type == "coupon") {
    var couponId = referrer.match(Config.redirectSiteCouponRegexp)[1];
    var clickedCoupon = _.detect(coupons, function(item) {
      return (_txt(item.id) == couponId);
    });
    this._views.NotificationBar.state = "couponActive";
    showArg.coupon = clickedCoupon;
    this._views.NotificationBar.show(showArg);
    return true;
  }

  if (type = "cashback") {
    this._views.NotificationBar.state = "cashbackActive";
    this._views.NotificationBar.show(showArg);
    return true;
  }
};


Main.prototype._matchExtrabuxRedirectURL = function(url) {
  function matchesStoreSite(url) {
    return _.any(Config.redirectSiteStoreRegexp, function(regexp) {
      return url.match(regexp);
    });
  }

  if (Utils.getHostname(url) == Config.redirectSite) {
    if (url.match(Config.redirectSiteCouponRegexp)) {
      logger.debug("Main controller -> checkOverrideDefaultBehaviour -> matched coupon");
      return "coupon"
    } else if (matchesStoreSite(url)) {
      logger.debug("Main controller -> checkOverrideDefaultBehaviour -> matched store");
      return "cashback";
    }
  }
  return null;
}


Main.prototype._loadCouponsAndShowBar = function(
    store, invalidReferrerFlag, fakeReferrer, showArg) {
  var self = this;
  var storeId = _txt(store.id);

  // We have to load the coupons first so that we can display how many there are.
  this._models.Coupon.loadListForMerchantId(storeId, {"on" : {
    "success": function(coupons) {

      if ( ! self._views.NotificationBar._visible) {
        var behaviorOverridden = false;

        if ( ! invalidReferrerFlag) {
          // The tab has not been restored from session cache (i.e., the
          // referrer is valid).
          behaviorOverridden =
            self.checkOverrideDefaultBehaviour(fakeReferrer, coupons, showArg);
        }

        if ( ! models.GlobalUser.activeUser) {
          behaviorOverridden = false;
        }

        if ( ! behaviorOverridden) {
          // Fall back to normal behaviour and show standard main state.
          self._views.NotificationBar.state = "main";
          self._views.NotificationBar.show(showArg);
        } else {
          // We're coming from the Extrabux site, so we must not generate
          // a click (it's already been generated).
          self._models.Click.generatedForCurrentSite = true;
          self._fakeReferrer = "";
        }
      }
    },
    "error": function() {}
  }});
};


Main.prototype.tabUnselected = function() {
  // Hide all the panels when the tab is unselected.
  this._views.UserAuth.hide();
  this._views.CouponList.hide();
  this._views.Restrictions.hide();
};


Main.prototype.restrictionsClicked = function() {
  var view = this._views.Restrictions;

  // Hide the coupon list, so that the panels do not overlap.
  this._views.CouponList.hide();

  if (view._visible) {
    view.hide();
  } else {
   view.show();
  }
};


Main.prototype.cashbackBtnClicked = function() {
  logger.debug("Main controller -> cashback button clicked");

  this._controllers.UserAuth.delayedActions.clear();

  this._views.CouponList.hide();
  this._views.Restrictions.hide();
  if (this._views.UserAuth._visible) {
    // Just hide the view.
    this._views.UserAuth.hide();
    return;
  }

  var user = models.GlobalUser.activeUser;
  if (!user) {
    // No user signed in => show signup page.
    this._views.UserAuth.state = "signup";
    this._views.UserAuth.show();
    var self = this;
    this._controllers.UserAuth.delayedActions.addAction(function() {
      self._controllers.Main.cashbackBtnClicked();
    });
  } else {
    this._views.NotificationBar.state = "cashbackActive";
    this._views.NotificationBar.show();
    this._generateClick(true);
  }
};


Main.prototype.storeLinkClicked = function() {
  var store = this._models.Store.activeStore;
  var url = _txt(store.store_url);
  logger.debug("MainController -> storeLinkClicked -> " + url);
};


Main.prototype._generateClick = function(useUserId) {
  logger.debug("Main controller -> _generateClick");
  var store = this._models.Store.activeStore;
  var user = models.GlobalUser.activeUser;

  var self = this;

  var params = {
    merchant_id: _txt(store.id)
  };

  if (useUserId) {
    params.user_id = _txt(user.email);
  }

  this._models.Click.generateClick(
    params, {
      on: {
        success: function(response) {
          logger.debug("Main controller -> _generateClick -> success");
          var url = _txt(response.click.merchant_transfer_url);
          self.pingMerchantSite(url);
        },
        error: function() {}
      }
    });
};


Main.prototype.pingMerchantSite = function(url) {
  // Create a hidden iframe to load the page into.
  var iframe = this._browser.contentDocument.createElement("iframe");
  iframe.width = 1;
  iframe.height = 1;
  iframe.type = "content";
  this._browser.contentDocument.body.appendChild(iframe);

  var store = this._models.Store.activeStore;
  var self = this;

  function handleSuccess() {
    if (!iframe) {
      return;
    }
    iframe.parentNode.removeChild(iframe);
    iframe = null;
  }

  function _handlePageLoad(event) {
    logger.debug("Main controller -> _generateClick -> iframe loaded");
    // Wait for two seconds before reloading the page.
    self._window.setTimeout(handleSuccess, 2000);
  }

  // Try to open the page in an iframe.
  iframe.contentWindow.addEventListener("DOMContentLoaded", _handlePageLoad, false);

  // Some pages don't open properly in iframes so fall back to AJAX for those.
  this._window.setTimeout(function() {
    if (!iframe) {
      // Iframe approach worked, just return.
      return;
    }
    // Remove the listener (the timeout fired earlier than DOMContentLoaded, so we assume
    // the iframe approach did not work).
    if (iframe.contentWindow) {
      iframe.contentWindow.removeEventListener("DOMContentLoaded", _handlePageLoad, false);
    }
    // Fall back to AJAX request.
    self._get(_txt(store.shop_url),
      {},
      {"on": {
        "success": function(reponse) {
          logger.debug("Main controller -> _generateClick -> fall back to AJAX -> success");
          handleSuccess();
        },
        "error" : function(reason) {}
      }});
  }, 5000);

  logger.debug("MainController -> pingMerchantSite -> url: " + url);
  iframe.setAttribute("src", url);
};


Main.prototype.couponsBtnClicked = function() {
  logger.debug("Main controller -> coupons button clicked");

  this._controllers.UserAuth.delayedActions.clear();

  this._views.UserAuth.hide();
  if (this._views.CouponList._visible) {
    this._views.CouponList.hide();
  } else {
    this._views.CouponList.show();
  }
  this._views.Restrictions.hide();
};


Main.prototype.closeButtonClicked = function() {
  this.lastClosed = Utils.getHostname(this.currentURL);

  logger.debug("Main -> closeButtonClicked -> click generated? " +
    this._models.Click.generatedForCurrentSite);

  // Only generate click when there is an active store (there may not be
  // any active store, e.g. when welcome notification bar is shown).
  var store = this._models.Store.activeStore;
  if (store &&
      ! this._models.Click.generatedForCurrentSite &&
      _txt(store.plugin_continue_without_cashback) != "0") {
    this._generateClick(false);
  }

  this._controllers.UserAuth.delayedActions.clear();

  // Close all the panels for this tab.
  this._views.UserAuth.hide();
  this._views.CouponList.hide();
  this._views.Restrictions.hide();
};


Main.prototype.clickedOutsidePanel = function(event) {
  this._controllers.UserAuth.delayedActions.clear();

  this._views.UserAuth.hide();
  this._views.CouponList.hide();
  this._views.Restrictions.hide();
};


Main.prototype.logoClicked = function() {
  this._window.gBrowser.selectedTab = this._window.gBrowser.addTab(Config.logoLinkURL);
};
