
var EXPORTED_SYMBOLS = ["NotificationBar"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;

var XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var notificationBoxName = "extrabux";

Cu.import("resource://extrabux/modules/user.jsm");

Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/genericView.jsm");
Cu.import("resource://extrabux/modules/config.jsm");
Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu.import("resource://extrabux/modules/underscore.jsm");

// Localization;
Components.utils.import("resource://gre/modules/PluralForm.jsm");


function NotificationBar(window, browser, models, jquery) {
  NotificationBar.baseConstructor.call(this, window, browser, models, jquery);

  this._viewPrepared = false;

  this.containerElement = null;

  this.state = "main";

  this._autoHideTimeout = null;

  var self = this;

  this.stateMap = {
    "main": function(arg) {self._showMain(arg);},
    "couponActive": function(arg) {self._showCoupon(arg);},
    "cashbackActive": function(arg) {self._showCashback(arg);},
    "welcome": function(arg) {self._showWelcome(arg);},
  };

  this._attachStoreListeners();
}


InheritFrom(GenericView, NotificationBar);


NotificationBar.prototype._attachStoreListeners = function() {
  var self = this;
  this.listLoadingRef = this._models.Store.storeCache.listLoading.attach(function() {
    self.listLoading();
  });
  this.listLoadedRef = this._models.Store.storeCache.listLoaded.attach(function(who, storesJSON) {
    self.listLoaded(storesJSON);
  });
  this.listErrorRef = this._models.Store.storeCache.listError.attach(function(reason) {
    self.listError();
  });
};


NotificationBar.prototype.detachListeners = function() {
  this._models.Store.storeCache.listLoading.detach(this.listLoadingRef);
  this._models.Store.storeCache.listLoaded.detach(this.listLoadedRef);
  this._models.Store.storeCache.listError.detach(this.listErrorRef);
};


NotificationBar.prototype.listLoading = function() {
  // Don't show anything if the list is not yet loaded.
  logger.debug("NotificationBar view -> listLoading.");
  this._viewPrepared = false;
};


NotificationBar.prototype.listLoaded = function(stores) {
  logger.debug("NotificationBar view -> listLoaded.");
  this._viewPrepared = true;
};


NotificationBar.prototype.listError = function() {
  logger.debug("NotificationBar listError.");
};


NotificationBar.prototype._getNotificationBox = function() {
  var nb = this._window.gBrowser.getNotificationBox(this._browser);
  var notifCreated = false;
  var n;

  try {
    n = nb.getNotificationWithValue(notificationBoxName);
  } catch (e) {
    nb =  this._browser.getNotificationBox();
    n = nb.getNotificationWithValue(notificationBoxName);
  }

  if(!n) {
    var priority = nb.PRIORITY_WARNING_HIGH;
    nb.appendNotification("", notificationBoxName,
      'chrome://browser/skin/Info.png',
      priority, []);
    notifCreated = true;
  }
  var notif = nb.getNotificationWithValue(notificationBoxName);

  if (notifCreated) {
    // Override the default behavior of the notification box close button.
    var p = notif.ownerDocument.getAnonymousElementByAttribute(
      notif, "anonid", "details");
    this.containerElement = this.$(p);

    var btn = this.$(p).next();
    btn.attr("oncommand",
      "com.extrabux.extensions.Extrabux.getMVCForBrowserTab()." +
      "controllers.Main.closeButtonClicked();" + btn.attr("oncommand"));
    // Set the logo.
    this.$(".extrabux-notif-bar-logo").attr("src", Config.logoURL);
  }
  // Keep the notification live until we explicitly hide it
  // (i.e., don't hide on page load).
  notif.persistence = -1;

  return notif;
};


/**
 * Replace the inner content of the notification box with user-defined template.
 * @param selector: jQuery selector for the template.
 */
NotificationBar.prototype._replaceNotifBarContents = function(selector) {
  // We have to call this to set up this.containerElem (even if we don't
  // need the actual notif box element here).
  this._getNotificationBox();

  var box = this.$(selector).clone(true);
  box.removeAttr("id");

  this.containerElement.replaceWith(box);
  this.containerElement = box;

  return box;
};


NotificationBar.prototype._showMain = function () {
  var store = this._models.Store.activeStore;
  if ( ! store) {
    return;
  }

  var coupons = this._models.Coupon.activeCoupons;

  var noCashback = (_txt(store.cashback_flat) == "0" &&
    _txt(store.cashback_percent) == "0");
  var nCoupons = coupons.length;

  if (noCashback && nCoupons === 0) {
    // Don't show anything when there's nothing to show.
    return;
  }

  var notif = this._getNotificationBox();

  var sb = this.$("#extrabux-strings").get(0);

  this._setupMouseOverCloseButtonHandler(sb.getString("close-btn-continue-wo-cashback"));

  var box;
  if (noCashback) {
    box = this._replaceNotifBarContents("#extrabux-notifBar-coupons-only-main-template");
  } else if (nCoupons === 0) {
    box = this._replaceNotifBarContents("#extrabux-notifBar-cashback-only-main-template");
  } else {
    box = this._replaceNotifBarContents("#extrabux-notifBar-two-button-main-template");
  }

  var cashbackFormatted = Utils.getCashbackString(store);

  var pluralRule = parseInt(sb.getString("plural-rule"), 10);
  var _get = PluralForm.makeGetter(pluralRule)[0];

  var msg;
  if (noCashback) {
    msg = _.sprintf(
      _get(nCoupons, sb.getString("notif-bar-main-msg-coupons-only")),
      nCoupons);
  } else if (nCoupons === 0) {
    msg = _.sprintf(
      sb.getString("notif-bar-main-msg-cashback-only"),
      cashbackFormatted);
  } else {
    msg = _.sprintf(
      _get(nCoupons, sb.getString("notif-bar-main-msg-both")),
      cashbackFormatted, nCoupons);
  }

  box.find(".extrabux-notif-bar-message").text(msg + " ");
  box.find(".extrabux-notif-bar-store-link").text(_txt(store.name));
};


NotificationBar.prototype._setupMouseOverCloseButtonHandler = function(msg) {
  var self = this;
  var closeBtn = this.containerElement.next();
  closeBtn.hover(
    function() {
      var msgElem = self.containerElement.find(".extrabux-notif-bar-close-button-msg");
      msgElem.text(msg);
    },
    function() {
      var msgElem = self.containerElement.find(".extrabux-notif-bar-close-button-msg");
      msgElem.text("");
    });
};


NotificationBar.prototype._showCoupon = function(args) {
  if ( ! this._models.Store.activeStore) {
    return;
  }

  var coupon = args.coupon;

  var notif = this._getNotificationBox();

  var box = this._replaceNotifBarContents("#extrabux-notifBar-activated-template");

  var noCashback = (_txt(coupon.store.cashback_flat) == "0" &&
    _txt(coupon.store.cashback_percent) == "0");

  var cashbackFormatted = Utils.getCashbackString(coupon.store);

  var stringsBundle = this.$("#extrabux-strings").get(0);

  this._setupMouseOverCloseButtonHandler(stringsBundle.getString("close-btn-hide"));

  var msgCoupon, msgCouponCode;
  var code = _txt(coupon.code);
  if (code) {
    // Show coupon code.
    msgCoupon = stringsBundle.getString(
      "notifBar-coupon-message-first-part-coupon-code");
    msgCouponCode = " " + code + ". ";
  } else {
    // Tell the user that the coupon has been actvated.
    msgCoupon = stringsBundle.getString(
      "notifBar-coupon-message-first-part-coupon-activate");
    msgCouponCode = " ";
  }
  var msgEnd = "";
  if ( ! noCashback && GlobalUser.activeUser) {
    // Tell user about cash back (do not show if there is no
    // cashback or if the user is not logged in).
    msgEnd += _.sprintf(" " + stringsBundle.getString("notifBar-coupon-message"),
      cashbackFormatted);
  }

  box.find(".extrabux-notif-bar-message-first").text(msgCoupon);
  box.find(".extrabux-notif-bar-message-couponCode").text(msgCouponCode);
  box.find(".extrabux-notif-bar-message-end").text(msgEnd);

  this._handleRestrictions();
};


NotificationBar.prototype._showCashback = function(args) {
  if ( ! this._models.Store.activeStore) {
    return;
  }

  var notif = this._getNotificationBox();

  var box = this._replaceNotifBarContents("#extrabux-notifBar-activated-template");

  var coupons = this._models.Coupon.activeCoupons;
  logger.debug("CASHBACK COUPONS: " + coupons.length);
  if (coupons.length === 0) {
    // Hide the coupons button if there are none to show.
    this.containerElement.find(".extrabux-notif-bar-coupons-button").hide();
  }

  var stringsBundle = this.$("#extrabux-strings").get(0);
  this._setupMouseOverCloseButtonHandler(stringsBundle.getString("close-btn-hide"));

  var store = this._models.Store.activeStore;
  var cashback = Utils.getCashbackString(store);
  var msg = _.sprintf(stringsBundle.getString(
    "notif-bar-cashback-message"), cashback);

  box.find(".extrabux-notif-bar-message-end").text(msg);

  this._handleRestrictions();
};


NotificationBar.prototype._showWelcome = function(args) {
  var notif = this._getNotificationBox();

  var box = this._replaceNotifBarContents("#extrabux-notifBar-welcome-template");
};


NotificationBar.prototype._handleRestrictions = function() {
  var store = this._models.Store.activeStore;
  var restrictions = this.containerElement.find(".extrabux-notif-bar-message-restrictions");
  if ( ! _txt(store.restrictions)) {
    restrictions.hide();
  } else {
    restrictions.show();
  }
};


NotificationBar.prototype.getButtons = function() {
  if (!this.containerElement) {
    // This means we're trying to hide too soon (before her notification box
    // is actually created). It's actually a bug, well...sort of, anyway.
    // But better safe then sorry, so we check it here.
    return {};
  }
  var buttons = this.containerElement.find(".extrabux-notif-bar-button");
  var ret = {
    cashback: this.containerElement.find(".extrabux-notif-bar-cashback-button"),
    coupon: this.containerElement.find(".extrabux-notif-bar-coupons-button")
  };

  return ret;
};


NotificationBar.prototype.show = function(args) {
  args = args || {};

  this.stateMap[this.state](args);
  this._visible = true;

  var self = this;
  if (args.hideTimeout) {
    this._autoHideTimeout = this._window.setTimeout(function() {
      self.hide();
    }, args.hideTimeout);
  }
};


NotificationBar.prototype.hide = function() {
  if ( ! this._visible) {
    return;
  }

  var nb = this._window.gBrowser.getNotificationBox(this._browser);
  var notif = nb.getNotificationWithValue(notificationBoxName);
  if (notif) {
    nb.removeNotification(notif);
  }

  this._window.clearTimeout(this._autoHideTimeout);
  this._autoHideTimeout = null;

  this._visible = false;
};


NotificationBar.prototype.destroy = function() {
  this.hide();
  this.detachListeners();
  logger.debug("Destroyed notification bar");
};
