var EXPORTED_SYMBOLS = ["Options"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;
Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/genericView.jsm");


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu.import("resource://extrabux/modules/underscore.jsm");


var models = {};
Cu.import("resource://extrabux/modules/user.jsm", models);
Cu.import("resource://extrabux/modules/loginManager.jsm", models);


function Options(windowArg, browserArg, modelsArg, jquery) {
  Options.baseConstructor.call(this, windowArg, browserArg, modelsArg, jquery);

  this.state = "signin";

  var self = this;
  this.userLoggedInRef = models.GlobalUser.userLoggedIn.attach(function(who, user) {
    self._handleLoginEvent();
  });
}


InheritFrom(GenericView, Options);


Options.prototype._handleLoginEvent = function() {
  logger.debug("options -> _handleLoginEvent");
  var loginMgr = models.GlobalUser.activeLoginManager;
  if (loginMgr.id == "facebook") {
    this.state = "fb-signed-in";
  } else if (loginMgr.id == "bargainmatch") {
    this.state = "signed-in";
  } else if (loginMgr.id == "website") {
    this.state = "website-signed-in";
  }
  if (this._visible) {
    this.show();
  }
};


Options.prototype.updateError = function(error) {
   error = error || "";
   this.$(".extrabux-userauth-error").text(error);
};


Options.prototype._showSignin = function() {
  this.$(".extrabux-prefs-panel").hide();
  this.$("#extrabux-prefs-sign-in-panel").show();
};


Options.prototype._showSignup = function() {
  this.$(".extrabux-prefs-panel").hide();
  this.$("#extrabux-prefs-sign-up-panel").show();
};


Options.prototype._showSignedIn = function() {
  this.$(".extrabux-prefs-panel").hide();
  this.$("#extrabux-prefs-signedin-panel").show();
};

Options.prototype._showFBSignedIn = function() {
  this.$(".extrabux-prefs-panel").hide();
  this.$("#extrabux-prefs-fb-signedin-panel").show();
};

Options.prototype._showWebsiteSignedIn = function() {
  this.$(".extrabux-prefs-panel").hide();
  this.$("#extrabux-prefs-website-signedin-panel").show();
};


Options.prototype.getCredentials = function() {
  var panelSelector;
  if (this.state == "signin") {
    panelSelector = "#extrabux-prefs-sign-in-panel";
  } else if (this.state == "signup") {
    panelSelector = "#extrabux-prefs-sign-up-panel";
  }
  var email = escape(this.$(panelSelector + " .extrabux-inputs-email").val());
  var pwd = escape(this.$(panelSelector + " .extrabux-inputs-pwd").val());
  return {
    email: email,
    pwd: pwd
  };
};


Options.prototype.hide = function() {
  this._window.close();
};


Options.prototype.show = function() {
  if (this.state == "signin") {
    this._showSignin();
  } else if (this.state == "signup") {
    this._showSignup();
  } else if (this.state == "signed-in") {
    this._showSignedIn();
  } else if (this.state == "fb-signed-in") {
    this._showFBSignedIn();
  } else if (this.state == "website-signed-in") {
    this._showWebsiteSignedIn();
  } else {
    throw "UNKNOWN STATE";
  }
  this._visible = true;
};


Options.prototype.destroy = function() {
  this.hide();
  models.GlobalUser.userLoggedIn.detach(this.userLoggedInRef);
};
