var EXPORTED_SYMBOLS = ["Options"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);

var models = {};
Cu.import("resource://extrabux/modules/user.jsm", models);
Cu.import("resource://extrabux/modules/config.jsm");

Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;
Cu.import("resource://extrabux/modules/underscore.jsm");
Cu.import("resource://extrabux/modules/genericController.jsm");
Cu.import("resource://extrabux/modules/inheritFrom.jsm");


function Options(window, models, views, jquery) {
  Options.baseConstructor.call(this, window, null, models, views, jquery);
}


InheritFrom(GenericController, Options);


Options.prototype._signin = function(email, pwd) {
  var self = this;
  models.GlobalUser.activeLoginManager = "extrabux";
  models.GlobalUser.authenticate(email, pwd, {on: {
    success: function() {
      self._views.Options.hide();
    },
    error: function(reason) {
      self._views.Options.updateError(reason);
    }
  }});
};


Options.prototype.signinClicked = function() {
  var creds = this._views.Options.getCredentials();
  this._signin(creds.email, creds.pwd);
};


Options.prototype.signupClicked = function() {
  var creds = this._views.Options.getCredentials();

  models.GlobalUser.activeLoginManager = "extrabux";

  var self = this;
  models.GlobalUser.create(creds.email, creds.pwd, {"on" : {
    "success": function(user) {
      self._signin(creds.email, creds.pwd);
    },
    "error": function(reason) {
      self._views.Options.updateError(reason);
      self.delayedActions.clear();
    }
  }});
};


Options.prototype.logoutClicked = function() {
  models.GlobalUser.logout();
  this._views.Options.state = "signin";
  this._views.Options.show();
};


Options.prototype.signupLinkClicked = function() {
  this._views.Options.state = "signup";
  this._views.Options.updateError(null);
  this._views.Options.show();
};


Options.prototype.signinLinkClicked = function() {
  this._views.Options.state = "signin";
  this._views.Options.updateError(null);
  this._views.Options.show();
};


Options.prototype.forgotPwdLinkClicked = function() {
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
    getService(Ci.nsIWindowMediator);
  var mainWindow = wm.getMostRecentWindow("navigator:browser");
  mainWindow.gBrowser.selectedTab = mainWindow.gBrowser.addTab(Config.forgotPwdLink);
  this._views.Options.hide();
};


Options.prototype.facebookLinkClicked = function() {
  models.GlobalUser.activeLoginManager = "facebook";
  models.GlobalUser.authenticate(null, null, {on: {
    success: function(activeUser) {
      logger.debug("facebookLinkClicked -> email: " + activeUser.email);
    },
    error: function() {}
  }});
  this._views.Options.hide();
};


Options.prototype.onLoad = function() {
  if ( ! models.GlobalUser.activeLoginManager) {
    // No user signed in.
    this._views.Options.state = "signin";
  } else {
    var stateDict = {
      "extrabux": "signed-in",
      "website":  "website-signed-in",
      "facebook": "fb-signed-in"
    };
    // User is already logged in => show correct view.
    this._views.Options.state =
      stateDict[models.GlobalUser.activeLoginManager.id];
  }
  this._views.Options.show();
};

