
var EXPORTED_SYMBOLS = ["LocalStorage"];


var Ci = Components.interfaces;
var Cc = Components.classes;
var Cu = Components.utils;


Cu.import("resource://extrabux/modules/logging.jsm");

/**
 * Grants access to low-level DB API.
 */
function LocalStorage() {
  var file = Cc["@mozilla.org/file/directory_service;1"].
    getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
  file.append("extrabux.sqlite");

  var storageService = Components.classes["@mozilla.org/storage/service;1"].
    getService(Components.interfaces.mozIStorageService);

  this.mDBConn = storageService.openDatabase(file);
  this.mDBConn.executeSimpleSQL("CREATE TABLE IF NOT EXISTS data (key varchar(10), value BLOB, PRIMARY KEY(key))");
}


/* Get data for @key (asynchronously). */
LocalStorage.prototype.getAsync = function(key, callbackObj) {
  var statement = this.mDBConn.createStatement("SELECT * FROM data WHERE key = :key");
  statement.params.key = key;

  var accumulator = null;

  statement.executeAsync({
    handleResult: function(aResultSet) {
      accumulator = "";
      for (let row = aResultSet.getNextRow(); row; row = aResultSet.getNextRow()) {
        let value = row.getResultByName("value");
        accumulator += value;
        logger.info("LocalStorage -> getAsync -> accumulator: " + accumulator);
      }
    },

    handleError: function(aError) {
      callbackObj.on.error("Database error.")
    },

    handleCompletion: function(aReason) {
      if (aReason != Components.interfaces.mozIStorageStatementCallback.REASON_FINISHED) {
        logger.debug("LocalStorage getAsync handle completion -> ERROR.");
        //TODO
      } else {
        if ( ! accumulator) {
          callbackObj.on.success(undefined);
          return;
        }
        var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
        var valueJSON = null;
        try {
          var valueJSON = nativeJSON.decode(accumulator);
        } catch (e) {
          logger.error("Can't decode: " + accumulator + ". Exception: " + e);
        } finally {
          callbackObj.on.success(valueJSON);
        }
      }
    }
  });
};


/* Store @value data under @key key. */
LocalStorage.prototype.setAsync = function(key, value, callbackObj) {
  var statement = this.mDBConn.createStatement(
    "INSERT OR REPLACE INTO data(key, value) values(:key, :value)");
  var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
  statement.params.value = nativeJSON.encode(value);
  statement.params.key = key;
  statement.executeAsync({
    handleResult: function(aResultSet) {
    },
    handleError: function(aError) {
      if (callbackObj) {
        callbackObj.on.error("Database error.")
      }
    },
    handleCompletion: function(aReason) {
      if (aReason != Components.interfaces.mozIStorageStatementCallback.REASON_FINISHED) {
        //TODO
      } else{
        if (callbackObj) {
          callbackObj.on.success();
        }
      }
    }
  });
}


/* Delete value indentified by @key key. */
LocalStorage.prototype.deleteKeyAsync = function(key, callbackObj) {
  var statement = this.mDBConn.createStatement("DELETE FROM data where key = :key");
  statement.params.key = key;
  statement.executeAsync({
    handleResult: function(aResultSet) {
    },
    handleError: function(aError) {
      if (callbackObj) {
        callbackObj.on.error("Database error.")
      }
    },
    handleCompletion: function(aReason) {
      if (aReason != Components.interfaces.mozIStorageStatementCallback.REASON_FINISHED) {
        //TODO
      } else{
        if (callbackObj) {
          callbackObj.on.success();
        }
      }
    }
  });
}
