
var EXPORTED_SYMBOLS = ["GlobalUninstaller"];


var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu["import"]("resource://extrabux/modules/logging.jsm");
Cu["import"]("resource://extrabux/modules/config.jsm");
Cu["import"]("resource://extrabux/modules/inheritFrom.jsm");


Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var Uninstaller;
try {
  Cu.import("resource://gre/modules/AddonManager.jsm");
  Uninstaller = UninstallerNew;
} catch (e) {
  Uninstaller = UninstallerOld;
}


/**
 * This class shares the uninstall-related code for FF3.6 and FF4+.
 */
function GenericUninstaller() {
  this.beingUninstalled = false;
  this._browser = null;
  this._register();
}


GenericUninstaller.prototype.initialize = function(browser) {
  this._browser = browser;
};


GenericUninstaller.prototype._register = function() {
  var observerService = Cc["@mozilla.org/observer-service;1"].getService(
    Ci.nsIObserverService);
  observerService.addObserver(this, "profile-before-change", false);
};


GenericUninstaller.prototype.observe = function(subject, topic, data) {
  if (topic == "profile-before-change" && this.beingUninstalled) {
    logger.debug("Uninstaller -> uninstalling addon");
    try {
      this._uninstallHandler();
    } catch (e) {
      logger.error("GenericUninstaller -> error when uninstaling: " + e);
    }
  }
};


/**
 * Called right away when the user uninstalls the addon.
 */
GenericUninstaller.prototype._earlyUninstallHandler = function() {
  this._browser.selectedTab = this._browser.addTab(Config.goodbyeUninstallURL);
}

/**
 * Called before the browser is restarted afetr the addon is uninstalled.
 */
GenericUninstaller.prototype._uninstallHandler = function() {
  var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
  prefs = prefs.getBranch("extensions.");
  prefs.deleteBranch("extrabux@extrabux.com");
};


/**
 * Uninstall handler for FF 3.6.
 */
function UninstallerOld() {
  UninstallerOld.baseConstructor.call(this);
}


InheritFrom(GenericUninstaller, UninstallerOld);


UninstallerOld.prototype._register = function() {
  this.superClass._register.call(this);
  var observerService = Cc["@mozilla.org/observer-service;1"].getService(
    Ci.nsIObserverService);
  observerService.addObserver(this, "em-action-requested", false);
};


UninstallerOld.prototype.observe = function(subject, topic, data) {
  logger.debug("Uninstaller -> observe");

  this.superClass.observe.call(this, subject, topic, data);

  if (topic == "em-action-requested") {
    isCorrectAddon = (
      subject.QueryInterface(Ci.nsIUpdateItem).id == "extrabux@extrabux.com");

    if ((subject instanceof Ci.nsIUpdateItem) && isCorrectAddon) {
      if (data == "item-uninstalled") {
        this.beingUninstalled = true;
        this._earlyUninstallHandler();
      }
    }
  }
};



/**
 * uninstall handler for FF 4.0+.
 */
function UninstallerNew() {
  UninstallerNew.baseConstructor.call(this);
}


InheritFrom(GenericUninstaller, UninstallerNew);


UninstallerNew.prototype._register = function() {
  this.superClass._register.call(this);
  AddonManager.addAddonListener(this);
};


UninstallerNew.prototype.onUninstalling = function(addon) {
  if (addon.id == "extrabux@extrabux.com") {
    this.beingUninstalled = true;
    this._earlyUninstallHandler();
  }
};


UninstallerNew.prototype.onOperationCancelled = function(addon) {
  if (addon.id == "extrabux@extrabux.com") {
    if ((addon.pendingOperations & AddonManager.PENDING_UNINSTALL) != 0) {
      this.beingUninstalled = true;
      this._earlyUninstallHandler();
    }
  }
};


var GlobalUninstaller = new Uninstaller();


