
var EXPORTED_SYMBOLS = ["GlobalUser"];


var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
Cu.import("resource://extrabux/modules/underscore.jsm");

Cu.import("resource://extrabux/modules/event.jsm");
Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/config.jsm");
Cu.import("resource://extrabux/modules/xmlHttpWrapper.jsm");

Cu.import("resource://extrabux/modules/extrabuxLoginManager.jsm");
Cu.import("resource://extrabux/modules/extrabuxWebsiteLoginManager.jsm");
Cu.import("resource://extrabux/modules/facebookLoginManager.jsm");

var baseURL = Config.baseURL + "user/";


function User() {
  this.activeUser = null;

  this._activeLoginManager = null;

  // Is set in initializeLoginManagers.
  this._managers = {};

  this._firstLoginAttempt = true;

  this.userLoggedIn = new Event(this);
}


/**
 * Initialization routine for login managers. We cannot do this in the
 * constructor, because we don't have access to window or jQuery object there.
 */
User.prototype.initializeLoginManagers = function(window, $) {
  if ( ! _.isEmpty(this._managers)) {
    // Already initialized.
    return;
  }

  this._managers = {
    facebook: new FacebookLoginManager(window.gBrowser, $),
    extrabux: new ExtrabuxLoginManager(window.gBrowser, $),
    website: new ExtrabuxWebsiteLoginManager(window.gBrowser, $)
  };
};


User.prototype.__defineSetter__("activeLoginManager", function(mgrId) {
  if ( ! this._managers[mgrId]) {
    throw "Unsupported login manager: " + mgrId;
  }
  if (this._activeLoginManager !== this._managers[mgrId]) {
    this._activeLoginManager = this._managers[mgrId];
  }
});


User.prototype.__defineGetter__("activeLoginManager", function() {
  return this._activeLoginManager;
});


User.prototype.authenticate = function(username, password, callbackObj) {
  var self = this;
  this._activeLoginManager.authenticate(username, password, {on: {
    success: function(user) {
      self.handleSuccessfulLogin(user);
      callbackObj.on.success(user);
    },
    error: function(reason) {
      callbackObj.on.error(reason);
      self._activeLoginManager = null;
    }
  }});
};


User.prototype.handleSuccessfulLogin = function(user) {
  this.activeUser = user;

  var prefs = Utils.getPrefs();
  prefs.setCharPref("activeLoginManager", this._activeLoginManager.id);
  prefs.setCharPref("activeLoginEmail", _txt(user.email));

  this.userLoggedIn.notify(user);

  logger.debug("User -> authenticate -> success -> user: " + nativeJSON.encode(user));
};


User.prototype.create = function(username, password, callbackObj) {
  var self = this;
  this._activeLoginManager.create(username, password, {on: {
    success: function(user) {
      self.authenticate(username, password, callbackObj);
    },
    error: function(reason) {
      callbackObj.on.error(reason);
    }
  }});
};


User.prototype.logout = function() {
  this._activeLoginManager.logout();
  this._activeLoginManager = null;
  this.activeUser = null;
};


User.prototype.handleInitialLogin = function() {

  if ( ! this._firstLoginAttempt) { 
    return;
  }
  this._firstLoginAttempt = false;

  var self = this;

  var mgr = _.detect(this._managers, function(mgrObj, mgrId) {
    return mgrObj.usedForLastLogin();
  });

  if ( ! mgr) {
    logger.debug("User model -> no initial login found => check if we are" +
      "logged in to Extrabux (to get the email from there).");
    this.activeLoginManager = "website";
    this.authenticate(null, null, {on: {
      success: function() {},
      error: function() {}
    }});
    return;
  }

  var prefs = Utils.getPrefs();
  var email = prefs.getCharPref("activeLoginEmail");

  if ( ! email) {
    logger.error("User -> handleInitialLogin -> no active email!");
    return;
  }

  logger.warning("User model -> handleInitialLogin -> attempting to log in: " + mgr._id);

  mgr.initialLogin(email, {on: {
    success: function(user) {
      logger.debug("User model -> handleInitialLogin -> success");
      self._activeLoginManager = mgr;
      self.handleSuccessfulLogin(user);
    },
    error:   function() {
      logger.warning("User model -> handleInitialLogin -> error");
    }
  }});
};

var GlobalUser = new User();

