var EXPORTED_SYMBOLS = ["UserAuth"];

var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;


Cu.import("resource://extrabux/modules/logging.jsm");
Cu.import("resource://extrabux/modules/underscore.jsm");
Cu.import("resource://extrabux/modules/config.jsm");

Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/genericView.jsm");


function UserAuth(window, browser, models, jquery) {
  UserAuth.baseConstructor.call(this, window, browser, models, jquery);

  var self = this;
  this._models.Store.activeStoreChanged.attach(function(who, store) {
    self.updateSignup(store);
  });
  this.state = "signup";

  this._signupPanel = this._cloneTemplatePanel("#extrabux-userauth-signup-panel-template");
  this._signinPanel = this._cloneTemplatePanel("#extrabux-userauth-signin-panel-template");

  this._signinPanel.find(".extrabux-options-forgot-pwd-link").attr("href", Config.forgotPwdLink);
}


InheritFrom(GenericView, UserAuth);


UserAuth.prototype.updateSignup = function(store) {
  // Change the displayed store URL.
  var header = this._signupPanel.find(".extrabux-userauth-header-signup");

  var cashback = Utils.getCashbackString(store);

  var stringsBundle = this.$("#extrabux-strings").get(0);
  var msg = _.sprintf(stringsBundle.getString("userauth-signup-header"), cashback);
  header.text(msg);
};


/*
 * Show an error message (when the signup/signing was unsuccessful).
 */
UserAuth.prototype.updateError = function(error) {
  error = error || "";

  this._getPanel().find(".extrabux-userauth-error").text(error);
};


UserAuth.prototype._getPanel = function() {
  var panel = null;
  if (this.state == "signup") {
    panel = this._signupPanel;
  } else if (this.state == "signin") {
    panel = this._signinPanel;
  }
  if (!panel) {
    throw "Invalid UserAuth view state: " + this.state;
  }
  return panel;
};


UserAuth.prototype.getCredentials = function() {
  var panel = this._getPanel();
  var creds = {
    "email": panel.find(".extrabux-inputs-email").val(),
    "pwd": panel.find(".extrabux-inputs-pwd").val()
  };
  return creds;
};


/**
 * Show the signup panel.
 *
 * @param couponId  If not null/undefined (which happens when a not-logged-in
 *  user clicks a coupon button), we should show the "continue w/o cash back"
 *  link and set the id attribute of this link to @couponId (so that we can
 *  keep track of the clicked coupon).
 */
UserAuth.prototype._showSignup = function(couponId) {
  this._signupPanel.attr("hidden", false);
  this._signinPanel.get(0).hidePopup();

  var btns = this._views.NotificationBar.getButtons();
  if (btns.cashback.length > 0) {
    this._signupPanel.get(0).openPopup(btns.cashback.get(0), "after_start", -50, 0);
  } else {
    this._signupPanel.get(0).openPopup(btns.coupon.get(0), "after_start", -50, 0);
  }

  var bottomLink = this._signupPanel.find(".extrabux-userauth-continue");
  if (couponId) {
    bottomLink.show();
    bottomLink.find(".extrabux-userauth-continue-link").attr("id", couponId);
  } else {
    bottomLink.hide();
    bottomLink.find(".extrabux-userauth-continue-link").removeAttr("id");
  }

  var self = this;
  this._signupPanel.keypress(function(event) {
    if (event.which == "13") {
      // Enter pressed => emulate submit.
      self._signupPanel.find(".extrabux-userauth-submit").click();
    }
  });
};


/**
 * Returns coupon id if there is a coupon linked with this particular
 * displayed user auth dialog (happens when we display it because a coupon
 * button was pressed). If there is no such coupon id (when the dialog was
 * displayed because of cash back button being clicked), return null.
 */
UserAuth.prototype.getLinkedCouponId = function() {
  var continueLink = this._signupPanel.find(
    ".extrabux-userauth-continue .extrabux-userauth-continue-link");
  return continueLink.attr("id") || null;
};


UserAuth.prototype._hideSignup = function() {
  logger.debug("UserAuth view -> hide signup");
  this._signupPanel.get(0).hidePopup();
  this.updateError(null);
 };


UserAuth.prototype._showSignin = function() {
  this._signinPanel.attr("hidden", false);

  var btns = this._views.NotificationBar.getButtons();
  if (btns.cashback.length > 0) {
    this._signinPanel.get(0).openPopup(btns.cashback.get(0), "after_start", -50, 0);
  } else {
    this._signinPanel.get(0).openPopup(btns.coupon.get(0), "after_start", -50, 0);
  }

  var self = this;
  this._signinPanel.keypress(function(event) {
    if (event.which == "13") {
      // Enter pressed => emulate submit.
      self._signinPanel.find(".extrabux-userauth-submit").click();
    }
  });
};


UserAuth.prototype._hideSignin = function() {
  this._signinPanel.get(0).hidePopup();
};


UserAuth.prototype.show = function() {
  if (this.state == "signup") {
    this._showSignup.apply(this, arguments);
  } else if (this.state == "signin") {
    this._showSignin.apply(this, arguments);
  }
  this._visible = true;
};

/* Hide everything. */
UserAuth.prototype.hide = function() {
  if ( ! this._visible) {
    // Don't hide what's already hidden.
    return;
  }
  this._hideSignup();
  this._hideSignin();
  this._visible = false;
};


UserAuth.prototype.destroy = function() {
  this._signinPanel.remove();
  this._signinPanel = null;
  this._signupPanel.remove();
  this._signupPanel = null;
};
