
var EXPORTED_SYMBOLS = ["Utils"];


var Cc = Components.classes;
var Cu = Components.utils;
var Ci = Components.interfaces;


Cu.import("resource://extrabux/modules/config.jsm");
Cu.import("resource://extrabux/modules/underscore.jsm");
Cu.import("resource://extrabux/modules/logging.jsm");


var Utils = (function(my) {

  my.getCurrentWindowURL = function(window) {
    var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                       .getInterface(Components.interfaces.nsIWebNavigation)
                       .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                       .rootTreeItem
                       .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                       .getInterface(Components.interfaces.nsIDOMWindow);

      return mainWindow.content.location.protocol + "//" + mainWindow.content.location.hostname;
  };


  my.URLDomainEquals = function(url1, url2) {
    // Get the domain names.
    // Note that the regexp searches for at least two dots, so URLs such as
    // shop.com (i.e., without www prefix) will not match the regexp and
    // will stay unchanged (which is what we want).
    url1 = url1 || "";
    url2 = url2 || "";
    var domain1 = url1.replace(/.+\.(.+\..+)(\/)?/, "$1");
    var domain2 = url2.replace(/.+\.(.+\..+)(\/)?/, "$1");

    return (domain1 == domain2);
  };

  //TODO(Tom): What about uname:pwd@domain.com? It's not a big deal, but we
  //should probably consider those too.
  var hostnameRegExp = new RegExp('^(?:f|ht)tp(?:s)?://([^/]+)', 'im');

  my.getHostname = function(url) {
    url = url || "";
    var match = url.match(hostnameRegExp);
    if (match && match.length > 0) {
      return match[1].toString();
    } else {
      return "";
    }
  };

  my.getAddonVersionAsync = function(callbackFun) {
    try {
      // Firefox 4 and later; Mozilla 2 and later
      Cu["import"]("resource://gre/modules/AddonManager.jsm");
      AddonManager.getAddonByID("extrabux@extrabux.com", function(addon) {
        callbackFun(addon.version);
      });
    } catch (ex) {
      // Firefox 3.6 and before; Mozilla 1.9.2 and before
      var em = Cc["@mozilla.org/extensions/manager;1"].getService(
        Ci.nsIExtensionManager);
      var addon = em.getItemForID("extrabux@extrabux.com");
      callbackFun(addon.version);
    }
  };



  my.navigateToURL = function(window, url) {
    var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                       .getInterface(Components.interfaces.nsIWebNavigation)
                       .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                       .rootTreeItem
                       .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                       .getInterface(Components.interfaces.nsIDOMWindow);
    window.gBrowser.loadURI(url);
  };


  /* Helper method that simplifies working with JSON parsed from XML.*/
  my.text = function(obj) {
    if (typeof(obj["#text"]) !== "undefined") {
      return obj["#text"];
    } else {
      return obj["#cdata-section"];
    }
  };


  /* Get rid of "http://". */
  my.trimURL = function(url) {
    return _.ltrim(url, "http://");
  };


  my.getCouponDateFromString = function(dateStr) {
    // They're not sending us an ISO-formatted date => reformat and parse.
    var tmp = dateStr.replace(" ", "T");
    // Provided date is UTC.
    tmp = tmp + "Z";
    var date = new Date(tmp);
    return date;
  };


  my.formatExpirationString = function(dateStr, stringsBundle) {
    var date = my.getCouponDateFromString(dateStr);
    var pst = convertGMTtoPST(date);

    var template = stringsBundle.getString("restrictions-message");

    var res;
    if (date.getFullYear() > Config.expireDateLimit) {
      res = stringsBundle.getString("coupon-never-expires-message");
    } else {
      res = _.sprintf(template, pst);
    }
    return res;
  };


  my.getCashbackString = function(obj) {
    var cashbackPercent = parseFloat(my.text(obj.cashback_percent), 10);
    // Cash back is in cents => divide by 100.
    var cashbackFlat = parseInt(my.text(obj.cashback_flat), 10) / 100;
    var cashback;
    if (cashbackPercent) {
      cashback = cashbackPercent + "%";
    } else {
      cashback = Config.currencySign + _.sprintf("%.2f", cashbackFlat);
    }
    return cashback;
  };


  /**
   * Taken from: http://www-01.ibm.com/support/docview.wss?uid=swg21453432
   */
  function convertGMTtoPST(gmtDate) {

    var myDate = new Date(gmtDate);
    var myNewDate;

    /* Check if Daylight Savings is true. */
    var gmt = new Date();
    var ssm = new Date();
    var fsn = new Date();
    var aDay = 1000 * 60 * 60 * 24;

    ssm.setMonth(2); // Starts on Second Sunday in March
    ssm.setDate(1);

    var counter = 0;

    for(i=0; i<15; i++) {
      if( counter == 2 ) {
        break;
      }
      if( ssm.getDay() == 6 ) {
        counter ++;
      }
      ssm.setTime(ssm.getTime() + aDay);
    }

    fsn.setMonth(10); // Ends on First Sunday in November
    fsn.setDate(1);

    for(i=0; i<7; i++) {
      if( fsn.getDay() == 6 ) {
        break;
      }
      fsn.setTime(fsn.getTime() + aDay);
    }

    var isDLS = (gmt >= ssm && gmt < fsn);

    /* Check for Daylight Savings ends. */
    var Hours;

    if (isDLS) {
      Hours = 7;
    } else {
      Hours = 8;
    }

    myDate.setTime(myDate.getTime() - (myDate.getTimezoneOffset() + (Hours * 60)) * 60 * 1000);

    myNewDate = new Date((myDate.getTime()));

    var myYear = myNewDate.getUTCFullYear();
    var myMonth = myNewDate.getUTCMonth() + 1;
    var myDay = myNewDate.getUTCDate();

    var result = _.sprintf("%02d-%02d-%04d", myMonth, myDay, myYear);

    return result;
  }


  my.getPrefs = function() {
    var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
    prefs = prefs.getBranch("extensions.extrabux@extrabux.com.");
    return prefs;
  };


  /**
   * Parses the query string and look for a param with a given key.
   * @return value (decoded) of the parameter or "" if parameter has not
   *    been found
   */
  my.getQueryParamByName = function(queryString, name){
    // Escape the '[]' brackets, so that we can use @name in a reg exp.
    name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    // Regexp that matches the query param with key == @name.
    var regexS = "[\\?&]" + name + "=([^&#]*)";
    var regex = new RegExp(regexS);
    var results = regex.exec(queryString);
    if(results == null)
      return "";
    else
      return decodeURIComponent(results[1].replace(/\+/g, " "));
  }


  return my;

})({});



