EXPORTED_SYMBOLS = ["HTTPWrapper"];

var Ci = Components.interfaces;
var Cc = Components.classes;

Components.utils.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);

Components.utils.import("resource://extrabux/modules/underscore.jsm");
Components.utils.import("resource://extrabux/modules/xml2json.jsm");
Components.utils.import("resource://extrabux/modules/config.jsm");


/**
 * Base class for dealing with HTTP requests.
 **/
function HTTPWrapper() {
}


HTTPWrapper.prototype._get = function(baseURL, params, callbackObj, noDefaultParams) {
  var req = this._getXMLHTTPRequest();
  params = params || {};

  if (! noDefaultParams) {
    _.extend(params, Config.defaultGETParams);
  }

  fullUrl = appendQueryStringtoURL(params, baseURL);

  req.open('GET', fullUrl, true);

  logger.debug("XMLHTTPWrapper GET to: " + fullUrl);

  var self = this;
  req.onreadystatechange = function(aEvt) {
    self._handleStateChange(req, callbackObj, aEvt);
  };
  req.send(null);
};


function appendQueryStringtoURL(params, baseURL) {
  var fullUrl = baseURL;
  if ( ! _.isEmpty(params)) {
    logger.debug("PARAMS: " + nativeJSON.encode(params));
    // Check if there already are some params in the URL.
    fullUrl = _.includes(baseURL, "&") ? baseURL + "&" : baseURL + "?";
    // Append the params.
    _.each(params, function(value, key) {
      fullUrl += encodeURIComponent(key) + "=" + encodeURIComponent(value) + "&";
    });
    // Get rid of the last '&'.
    fullUrl = _.rtrim(fullUrl, "&");
  }
  return fullUrl;
}


HTTPWrapper.prototype._post = function(baseURL, getParams, postParams, callbackObj) {
  var req = this._getXMLHTTPRequest();

  getParams = getParams || {};
  _.extend(getParams, Config.defaultGETParams);

  fullUrl = appendQueryStringtoURL(getParams, baseURL);

  postParams = postParams || {};
  _.extend(postParams, Config.defaultPOSTParams);

  var paramString = "";
  _.each(postParams, function(value, key) {
    paramString += encodeURIComponent(key) + "=" + encodeURIComponent(value) + "&";
  });

  paramString = _.rtrim(paramString, "&");

  req.open('POST', fullUrl, true);
  req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  req.setRequestHeader("Content-length", paramString.length);
  req.setRequestHeader("Connection", "close");

  logger.debug("XMLHTTPWrapper POST to: " + fullUrl);
  logger.debug("XMLHTTPWrapper POST params: " + paramString);

  var self = this;
  req.onreadystatechange = function(aEvt) {
    self._handleStateChange(req, callbackObj, aEvt);
  };

  req.send(paramString);

};


HTTPWrapper.prototype._handleStateChange = function(req, callbackObj, evt) {
  if (req.readyState == 4) {
    if(req.status == 200) {
      callbackObj.on.success(req.responseText);
    } else {
      logger.debug("HTTPWrapper -> request error");
      callbackObj.on.error();
    }
  }
};


HTTPWrapper.prototype._getXMLHTTPRequest = function() {
  var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].
    createInstance(Components.interfaces.nsIXMLHttpRequest);
  return req;
};



